/*
 *  Linux driver for Comtrol SI/IA & DeviceMaster products.
 *
 *  nslink.h --- internal header file of the NSLink driver
 *
 *  Written by Grant Edwards, Comtrol Corp.  1995 - 2003.
 * 
 *  Copyright (C) 1995 - 2003 by Comtrol, Inc.
 * 
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _NSLINK_H
#define _NSLINK_H

struct si_config {
  int	line;
  int	flags;
  int	closing_wait;
  int	close_delay;
  int	port;
};

struct si_load_struct {
  unsigned long	offset;
  unsigned long	size;
  unsigned char	*buf;
};

/*
 * Maximum number of boxes
 */

// kdr  Kernel 2.6 page problem occurs a lot when this is a big num
//#define MAX_NRP_SIS		64
#define MAX_NRP_SIS		4

/*
 * Maximum number of ports for a box
 */
#define MAX_NRP_PORTS	32

struct si_box_config {
  int		boxid;
  int           base_port;
  int		num_ports;
  int		state;
  unsigned char	ether_addr[6];
  char		interface[16];
  unsigned long ip_addr;
  unsigned char	rsmode[MAX_NRP_PORTS];
  int           link_timeout;
  int           arch_id;
  unsigned char scan_period;
  int           low_latency[MAX_NRP_PORTS];
  int           inter_char_timeout[MAX_NRP_PORTS];
};

struct si_box_report {
  unsigned char	ether_addr[6];
  int		product_id;
  int		flags;
};

/*
 * MAC connected device structure
 */
struct mac_mode_params {
  int           boxid;
};

/*
 * TCP connected device structures.
 */
struct tcp_mode_params {
  int           boxid;
  int           sk;
  int           rbuf_size;
  unsigned char	*rbuf;
  int           wbuf_size;
  unsigned char	*wbuf;
};


typedef struct {
  unsigned char	cmd;
  unsigned char	index;
  unsigned short	length;
  unsigned long	addr;
} __attribute__((packed)) tToolRequest;

#define LOAD_TOOLREQ	1
#define GET_TOOLREQ	2
#define GOTO_TOOLREQ	3
#define MOVE_TOOLREQ	4
#define ID_TOOLREQ	5
#define SET_TOOLREQ	6
#define ENV_TOOLREQ	7
#define PING_TOOLREQ	8
#define DATA_TOOLREQ	9
#define SET_IP_TOOLREQ  10
#define ERASE_IP_TOOLREQ 11
#define DISABLE_IP_TOOLREQ 12
#define RESET_TOOLREQ   13

struct mac_id_response 
{
  unsigned char	MacAddr[6]        __attribute__((packed));
  unsigned char flags             __attribute__((packed));
  unsigned char unused[3]         __attribute__((packed));
  unsigned long   ModelId         __attribute__((packed));
  unsigned char   ArchId          __attribute__((packed));
  unsigned char   NumPorts        __attribute__((packed));
  unsigned short  ProgramId       __attribute__((packed));
  unsigned char   BoardRev        __attribute__((packed));
  unsigned char   BootRevMajor    __attribute__((packed));
  unsigned char   BootRevMinor    __attribute__((packed));
  unsigned char   NumApps         __attribute__((packed));
  struct 
  {
    unsigned short AppId        __attribute__((packed));
    unsigned char  AppRevMajor  __attribute__((packed));
    unsigned char  AppRevMinor  __attribute__((packed));
  }AppRecord[0]                __attribute__((packed));
  unsigned char   UserDataCount;
  unsigned char   UserData[0];
};

typedef struct
{
  unsigned long	IpAddr            __attribute__((packed));
  unsigned long	DefaultRoute      __attribute__((packed));
  unsigned long	ServerIpAddr      __attribute__((packed));
  unsigned long	SubnetMask        __attribute__((packed));
  unsigned long	AvailMemAddr      __attribute__((packed));
  unsigned long	AvailMemLength    __attribute__((packed));
  unsigned char	Version[64]       __attribute__((packed));
  unsigned char	MacAddr[6]        __attribute__((packed));
  unsigned long   ModelId         __attribute__((packed));
  unsigned char   ArchId          __attribute__((packed));
  unsigned char   NumPorts        __attribute__((packed));
  unsigned short  ProgramId       __attribute__((packed));
  unsigned char   BoardRev        __attribute__((packed));
  unsigned char   BootRevMajor    __attribute__((packed));
  unsigned char   BootRevMinor    __attribute__((packed));
  unsigned char   NumApps         __attribute__((packed));
  struct 
  {
    unsigned short AppId        __attribute__((packed));
    unsigned char  AppRevMajor  __attribute__((packed));
    unsigned char  AppRevMinor  __attribute__((packed));
  }AppRecord[0]                __attribute__((packed));
  unsigned char   UserDataCount;
  unsigned char   UserData[0];
} __attribute__((packed)) tToolResponse;

typedef struct
{
  unsigned long	IpAddr            __attribute__((packed));
  unsigned long	DefaultRoute      __attribute__((packed));
  unsigned long	ServerIpAddr      __attribute__((packed));
  unsigned long	SubnetMask        __attribute__((packed));
  unsigned long	AvailMemAddr      __attribute__((packed));
  unsigned long	AvailMemLength    __attribute__((packed));
  unsigned char	Version[64]       __attribute__((packed));
  unsigned char	MacAddr[6]        __attribute__((packed));
}  __attribute__((packed)) tOldToolResponse;;

/*
 *  SI flags
 */
#define SI_CALLOUT_NOHUP        0x00000001
#define SI_FORCE_CD		0x00000002
#define SI_HUP_NOTIFY	        0x00000004
#define SI_SPLIT_TERMIOS	0x00000008
#define SI_SPD_MASK		0x00000070
#define SI_SPD_HI		0x00000010 /* Use 56000 instead of 38400 bps */
#define SI_SPD_VHI		0x00000020 /* Use 115200 instead of 38400 bps*/
#define SI_SPD_SHI		0x00000030 /* Use 230400 instead of 38400 bps*/
#define SI_SPD_WARP	        0x00000040 /* Use 460800 instead of 38400 bps*/
#define SI_SAK		        0x00000080
#define SI_SESSION_LOCKOUT	0x00000100
#define SI_PGRP_LOCKOUT	        0x00000200
	
#define SI_FLAGS		0x000003FF

#define SI_USR_MASK 0x0071	/* Legal flags that non-privileged
				 * users can set or reset */

/*
 * For closing_wait and closing_wait2
 */
#define SI_CLOSING_WAIT_NONE	65535
#define SI_CLOSING_WAIT_INF		0

/*
 *  SI state machine defines
 */
#define SI_STATE_RESET		 0
#define SI_STATE_BOOTING	 1
#define SI_STATE_BOOTED		 2
#define SI_STATE_CONNECTING	 3
#define SI_STATE_CONNECTING_WAIT 4
#define SI_STATE_CONNECTED	 5
#define SI_STATE_DEAD		 6

typedef struct {
  int sock;
  int msTimeout;
} tTcpTxQueueWait;

/*
 *   NSLinkctl device ioctls supported
 */
#define SI_LOAD_CODE4_8 	0x00565301
#define SI_GET_CONFIG	  	0x00565302
#define SI_SET_CONFIG		0x00565303
#define SI_GET_CTL_MAJOR	0x00565304
#define SI_SET_BOX_CONFIG  	0x00565305
#define SI_GET_BOX_CONFIG	0x00565306
#define SI_GET_BOX_REPORT	0x00565307
#define SI_GET_PROBLEM_BOXES	0x00565308
#define SI_DEBUG_MISC		0x00565309
#define SI_SET_TCP	        0x0056530A
#define SI_LOAD_CODE2	        0x0056530B
#define SI_SET_MAC              0x0056530C
#define SI_TCP_TXQUEUE_WAIT     0x0056530D
#define SI_FORCE_PORT_RELEASE   0x0056530E
#define SI_RELEASE_MODULE       0x0056530F

/*
 *   NSLinkctl device ioctls supported
 */
#define SI_RSMODE_232  0x32
#define SI_RSMODE_422  0x22
#define SI_RSMODE_485  0x85
#define SI_RSMODE_OFF  0xde

#endif // _NSLINK_H
